# -*- coding: utf-8 -*-
"""Client used or interacting with our language identification api"""

import json
from .generic_api_client import GenericApiClient
from .models import LanguageIdentificationJob, LanguageIdentificationResult

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


class LanguageIdentificationClient(GenericApiClient):
    """Client for interacting with the Rev AI language identification api"""

    # Default version of Rev AI language identification api
    api_version = 'v1'

    # Default api name of Rev AI language identification api
    api_name = 'languageid'

    def __init__(self, access_token):
        """Constructor

        :param access_token: access token which authorizes all requests and links them to your
                             account. Generated on the settings page of your account dashboard
                             on Rev AI.
        """

        GenericApiClient.__init__(self, access_token, self.api_name, self.api_version,
                                  LanguageIdentificationJob.from_json,
                                  LanguageIdentificationResult.from_json)

    def submit_job_url(
            self,
            media_url,
            metadata=None,
            callback_url=None,
            delete_after_seconds=None,
            source_config=None,
            notification_config=None):
        """Submit media as a URL for language identification.
        The audio data is downloaded from the URL.

        :param media_url: web location of the media file
        .. deprecated:: 2.16.0
            Use source_config instead
        :param metadata: info to associate with the language identification job
        :param callback_url: callback url to invoke on job completion as a webhook
        .. deprecated:: 2.16.0
                Use notification_config instead
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param source_config: CustomerUrlData object containing url of the source media and
            optional authentication headers to use when accessing the source url
        :param notification_config: CustomerUrlData object containing the callback url to
            invoke on job completion as a webhook and optional authentication headers to use when
            calling the callback url
        :returns: raw response data
        :raises: HTTPError
        """
        payload = self.create_payload_with_source(media_url, source_config, metadata, callback_url,
                                                  delete_after_seconds, notification_config)

        return self._submit_job(payload)

    def submit_job_local_file(
            self,
            filename,
            metadata=None,
            callback_url=None,
            delete_after_seconds=None,
            notification_config=None):
        """Submit a local file for language identification.
        Note that the content type is inferred if not provided.

        :param filename: path to a local file on disk
        :param metadata: info to associate with the language identification job
        :param callback_url: callback url to invoke on job completion as a webhook
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param notification_config: CustomerUrlData object containing the callback url to
            invoke on job completion as a webhook and optional authentication headers to use when
            calling the callback url
        :returns: raw response data
        :raises: HTTPError
        """
        if not filename:
            raise ValueError('filename must be provided')

        payload = self._enhance_payload({}, metadata, callback_url, delete_after_seconds,
                                        notification_config)

        with open(filename, 'rb') as f:
            files = {
                'media': (filename, f),
                'options': (None, json.dumps(payload, sort_keys=True))
            }

            response = self._make_http_request(
                "POST",
                urljoin(self.base_url, 'jobs'),
                files=files
            )

        return LanguageIdentificationJob.from_json(response.json())

    def get_result_json(self, id_):
        """Get result of a language identification job as json.

        :param id_: id of job to be requested
        :returns: job result data as raw json
        :raises: HTTPError
        """
        return self._get_result_json(id_, {})

    def get_result_object(self, id_):
        """Get result of a language identification job as LanguageIdentificationResult object.

        :param id_: id of job to be requested
        :returns: job result data as LanguageIdentificationResult object
        :raises: HTTPError
        """
        return self._get_result_object(id_, {})



# -*- coding: utf-8 -*-
"""Class for Customer Url Data model"""


class CustomerUrlData:
    """Object used to contain a url provided by the customer along with any other related
    information, such as authentication headers."""

    def __init__(self, url, auth_headers=None):
        """Constructor
        :param url: customer provided url
        :param auth_headers: optional customer provided headers to access the url
        Only the "Authorization" header is currently supported
        This should be a dictionary in the form {"header": "scheme token"}
        For example: {"Authorization": "Bearer $BEARER_TOKEN"}
        """
        self.url = url
        self.auth_headers = auth_headers

    def to_dict(self):
        """Returns the raw form of the url data object as the api
        expects them"""
        dict_result = {'url': self.url}
        if self.auth_headers:
            dict_result['auth_headers'] = self.auth_headers
        return dict_result



# -*- coding: utf-8 -*-
"""Speaker name model"""


class SpeakerName:
    """SpeakerName object to provide a clean way
    to pass speaker names to human transcription"""

    def __init__(self, display_name):
        """Constructor

        :param display_name: human readable string representing the speaker's name
        """
        self.display_name = display_name

    def to_dict(self):
        """Returns the raw form of the speaker name as the api
        expects it"""
        return {'display_name': self.display_name}



# -*- coding: utf-8 -*-
"""Enum for caption content types"""

from enum import Enum


class CaptionType(Enum):
    SRT = "application/x-subrip"
    VTT = "text/vtt"

    @classmethod
    def from_string(cls, status):
        return cls[status.upper()]



# -*- coding: utf-8 -*-
"""Unit tests for Rev Ai Utils"""

from src.rev_ai.utils import _process_vocabularies
from src.rev_ai.models import CustomVocabulary

phrases = ["Patrick Henry Winston", "Noam Chomsky"]
other_phrases = ['Steve Jobs']


class TestUtils:
    def test_process_vocabularies_with_custom_vocab_dict(self):
        customvocabs = [{'phrases': phrases}]

        processed_vocabs = _process_vocabularies(customvocabs)

        assert processed_vocabs == [{'phrases': phrases}]

    def test_process_vocabularies_with_CustomVocabulary_instance(self):
        customvocabs = [CustomVocabulary(phrases)]

        processed_vocabs = _process_vocabularies(customvocabs)

        assert processed_vocabs == [{'phrases': phrases}]

    def test_process_vocabularies_with_mixed_input(self):
        customvocabs = [CustomVocabulary(phrases), {'phrases': other_phrases}]

        processed_vocabs = _process_vocabularies(customvocabs)

        assert processed_vocabs == [{'phrases': phrases}, {'phrases': other_phrases}]

    def test_process_vocabularies_with_empty_list(self):
        processed_vocabs = _process_vocabularies([])

        assert processed_vocabs == []



# -*- coding: utf-8 -*-
"""Media Config Model """


class MediaConfig:
    def __init__(self, content_type='audio/*', layout=None, rate=None,
                 audio_format=None, channels=None):
        """Constructor for MediaConfig class

        :param content_type (optional): string containing content type. Default
            will resort to 'audio/*'
        :param layout (optional): layout of audio
        :param rate (optional): sampling rate of audio
        :param audio_format (optional): format of audio
        :param channels (optional): the number of channels the audio has
        """
        self.content_type = content_type
        self.layout = layout
        self.rate = rate
        self.format = audio_format
        self.channels = channels

    def get_content_type_string(self):
        """Returns the content type and params as a string for the websocket
            connection
        """
        return self.content_type + \
            (';layout={}'.format(self.layout) if self.layout else '') + \
            (';rate={}'.format(self.rate) if self.rate else '') + \
            (';format={}'.format(self.format) if self.format else '') + \
            (';channels={}'.format(self.channels) if self.channels else '')



# -*- coding: utf-8 -*-
"""Transcript model"""


class Transcript:
    def __init__(self, monologues):
        """
        :param monologues: list of monologues included in output
        """
        self.monologues = monologues

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.monologues, other.monologues))
        return False

    def to_dict(self):
        """Returns the raw form of the transcript as the api
        returns them"""
        return {'monologues': [monologue.to_dict() for monologue in self.monologues]}

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls([Monologue.from_json(monologue) for monologue in json.get('monologues', [])])


class Monologue:
    def __init__(self, speaker, elements, speaker_info=None):
        """
        :param speaker: speaker identified for this monologue
        :param elements: list of elements spoken in this monologue
        :param speaker_info: information about the speaker if available
        """
        self.speaker = speaker
        self.elements = elements
        self.speaker_info = speaker_info

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.elements, other.elements)) \
                and self.speaker == other.speaker \
                and self.speaker_info == other.speaker_info
        return False

    def to_dict(self):
        """Returns the raw form of the monologue as the api
        returns them"""
        json = {'speaker': self.speaker,
                'elements': [element.to_dict() for element in self.elements]}
        if self.speaker_info:
            json['speaker_info'] = self.speaker_info.to_dict()
        return json

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        speaker_info = None
        if json.get('speaker_info') is not None:
            speaker_info = SpeakerInfo.from_json(json['speaker_info'])
        return cls(
            json['speaker'],
            [Element.from_json(element) for element in json.get('elements', [])],
            speaker_info)


class SpeakerInfo:
    def __init__(self, id_, display_name):
        """
        :param id_: speaker id identified for this monologue
        :param display_name: Human readable name of the speaker if available
        """
        self.id = id_
        self.display_name = display_name

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.id == other.id and self.display_name == other.display_name
        return False

    def to_dict(self):
        """Returns the raw form of the monologue as the api
        returns them"""
        return {'id': self.id,
                'display_name': self.display_name}

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['id'],
            json['display_name'])


class Element:
    def __init__(self, type_, value, timestamp, end_timestamp, confidence):
        """
        :param type_: type of element: text, punct, or unknown
        :param value: value of the element
        :param timestamp: time at which this element starts in the audio
        :param end_timestamp: time at which this element ends in the audio
        :param confidence: confidence in this output
        """
        self.type_ = type_
        self.value = value
        self.timestamp = timestamp
        self.end_timestamp = end_timestamp
        self.confidence = confidence

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    def to_dict(self):
        """Returns the raw form of the element as the api
        returns them"""
        return {'type': self.type_, 'value': self.value, 'ts': self.timestamp,
                'end_ts': self.end_timestamp, 'confidence': self.confidence}

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['type'],
            json['value'],
            json.get('ts'),
            json.get('end_ts'),
            json.get('confidence'))



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev AI"""

import json
from .models import Account, CaptionType, Job, Transcript
from .baseclient import BaseClient
from . import utils

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


class RevAiAPIClient(BaseClient):
    """Client which implements Rev AI API

    Note that HTTPErrors can be thrown by methods of the API client. The HTTP
    response payload attached to these error is a problem details. The problem
    details information is represented as a JSON object with error specific
    properties that help to troubleshoot the problem.

    Problem details are defined at https://tools.ietf.org/html/rfc7807.
    """

    # Default version of Rev AI
    version = 'v1'

    # Default base url for Rev AI
    base_url = 'https://api.rev.ai/speechtotext/{}/'.format(version)

    # Rev AI transcript format
    rev_json_content_type = 'application/vnd.rev.transcript.v1.0+json'

    def __init__(self, access_token):
        """Constructor

        :param access_token: access token which authorizes all requests and links them to your
                             account. Generated on the settings page of your account dashboard
                             on Rev AI.
        """

        BaseClient.__init__(self, access_token)

    def submit_job_url(
            self,
            media_url=None,
            metadata=None,
            callback_url=None,
            skip_diarization=False,
            skip_punctuation=False,
            speaker_channels_count=None,
            custom_vocabularies=None,
            filter_profanity=False,
            remove_disfluencies=False,
            delete_after_seconds=None,
            language=None,
            custom_vocabulary_id=None,
            transcriber=None,
            verbatim=None,
            rush=None,
            test_mode=None,
            segments_to_transcribe=None,
            speaker_names=None,
            source_config=None,
            notification_config=None):
        """Submit media given a URL for transcription.
        The audio data is downloaded from the URL
        :param media_url: web location of the media file
        .. deprecated:: 2.16.0
            Use source_config instead
        :param metadata: info to associate with the transcription job
        :param callback_url: callback url to invoke on job completion as a webhook
        .. deprecated:: 2.16.0
            Use notification_config instead
        :param skip_diarization: should Rev AI skip diarization when transcribing this file
        :param skip_punctuation: should Rev AI skip punctuation when transcribing this file
        :param speaker_channels_count: the number of speaker channels in the
            audio. If provided the given audio will have each channel
            transcribed separately and each channel will be treated as a single
            speaker. Valid values are integers 1-8 inclusive.
        :param custom_vocabularies: a collection of phrase dictionaries.
            Including custom vocabulary will inform and bias the speech
            recognition to find those phrases. Each dictionary should consist
            of a key "phrases" which maps to a list of strings, each of which
            represents a phrase you would like the speech recognition to bias
            itself toward. Cannot be used with the custom_vocabulary_id parameter.
        :param filter_profanity: whether to mask profane words
        :param remove_disfluencies: whether to exclude filler words like "uh"
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param language: specify language using the one of the supported ISO 639-1 (2-letter) or
            ISO 639-3 (3-letter) language codes as defined in the API Reference
        :param custom_vocabulary_id: The id of a pre-completed custom vocabulary
            submitted through the custom vocabularies api. Cannot be used with the
            custom_vocabularies parameter.
        :param transcriber: type of transcriber to use to transcribe the media file
        :param verbatim: Only available with "human" transcriber.
            Whether human transcriber transcribes every syllable.
        :param rush: Only available with "human" transcriber.
            Whether job is given higher priority to be worked on sooner for higher pricing.
        :param test_mode: Only available with "human" transcriber.
            Whether human transcription job is mocked and no transcription actually happens.
        :param segments_to_transcribe: Only available with "human" transcriber.
            Sections of transcript needed to be transcribed.
        :param speaker_names: Only available with "human" transcriber.
            Human readable names of speakers in the file.
        :param source_config: CustomerUrlData object containing url of the source media and
            optional authentication headers to use when accessing the source url
        :param notification_config: CustomerUrlData object containing the callback url to
            invoke on job completion as a webhook and optional authentication headers to use when
            calling the callback url
        :returns: raw response data
        :raises: HTTPError
        """
        payload = self._create_job_options_payload(media_url, metadata, callback_url,
                                                   skip_diarization, skip_punctuation,
                                                   speaker_channels_count,
                                                   custom_vocabularies, filter_profanity,
                                                   remove_disfluencies, delete_after_seconds,
                                                   language, custom_vocabulary_id, transcriber,
                                                   verbatim, rush, test_mode,
                                                   segments_to_transcribe, speaker_names,
                                                   source_config, notification_config)

        response = self._make_http_request(
            "POST",
            urljoin(self.base_url, 'jobs'),
            json=payload
        )

        return Job.from_json(response.json())

    def submit_job_local_file(
            self,
            filename,
            metadata=None,
            callback_url=None,
            skip_diarization=False,
            skip_punctuation=False,
            speaker_channels_count=None,
            custom_vocabularies=None,
            filter_profanity=False,
            remove_disfluencies=False,
            delete_after_seconds=None,
            language=None,
            custom_vocabulary_id=None,
            transcriber=None,
            verbatim=None,
            rush=None,
            test_mode=None,
            segments_to_transcribe=None,
            speaker_names=None,
            notification_config=None):
        """Submit a local file for transcription.
        Note that the content type is inferred if not provided.

        :param filename: path to a local file on disk
        :param metadata: info to associate with the transcription job
        :param callback_url: callback url to invoke on job completion as a webhook
        .. deprecated:: 2.16.0
                Use notification_config instead
        :param skip_diarization: should Rev AI skip diarization when transcribing this file
        :param skip_punctuation: should Rev AI skip punctuation when transcribing this file
        :param speaker_channels_count: the number of speaker channels in the
            audio. If provided the given audio will have each channel
            transcribed separately and each channel will be treated as a single
            speaker. Valid values are integers 1-8 inclusive.
        :param custom_vocabularies: a collection of phrase dictionaries.
            Including custom vocabulary will inform and bias the speech
            recognition to find those phrases. Each dictionary has the key
            "phrases" which maps to a list of strings, each of which represents
            a phrase you would like the speech recognition to bias itself toward.
            Cannot be used with the custom_vocabulary_id parameter
        :param filter_profanity: whether to mask profane words
        :param remove_disfluencies: whether to exclude filler words like "uh"
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param language: specify language using the one of the supported ISO 639-1 (2-letter) or
            ISO 639-3 (3-letter) language codes as defined in the API Reference
        :param custom_vocabulary_id: The id of a pre-completed custom vocabulary
            submitted through the custom vocabularies api. Cannot be used with the
            custom_vocabularies parameter.
        :param transcriber: type of transcriber to use to transcribe the media file
        :param verbatim: Only available with "human" transcriber.
            Whether human transcriber transcribes every syllable.
        :param rush: Only available with "human" transcriber.
            Whether job is given higher priority to be worked on sooner for higher pricing.
        :param test_mode: Only available with "human" transcriber.
            Whether human transcription job is mocked and no transcription actually happens.
        :param segments_to_transcribe: Only available with "human" transcriber.
            Sections of transcript needed to be transcribed.
        :param speaker_names: Only available with "human" transcriber.
            Human readable names of speakers in the file.
        :param notification_config: CustomerUrlData object containing the callback url to
            invoke on job completion as a webhook and optional authentication headers to use when
            calling the callback url
        :returns: raw response data
        :raises: HTTPError, ValueError
        """
        if not filename:
            raise ValueError('filename must be provided')

        payload = self._create_job_options_payload(None, metadata, callback_url,
                                                   skip_diarization, skip_punctuation,
                                                   speaker_channels_count,
                                                   custom_vocabularies, filter_profanity,
                                                   remove_disfluencies, delete_after_seconds,
                                                   language, custom_vocabulary_id, transcriber,
                                                   verbatim, rush, test_mode,
                                                   segments_to_transcribe, speaker_names, None,
                                                   notification_config)

        with open(filename, 'rb') as f:
            files = {
                'media': (filename, f),
                'options': (None, json.dumps(payload, sort_keys=True))
            }

            response = self._make_http_request(
                "POST",
                urljoin(self.base_url, 'jobs'),
                files=files
            )

        return Job.from_json(response.json())

    def get_job_details(self, id_):
        """View information about a specific job.
        The server will respond with the status and creation date.

        :param id_: id of the job to be requested
        :returns: Job object if available
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}'.format(id_))
        )

        return Job.from_json(response.json())

    def get_list_of_jobs(self, limit=None, starting_after=None):
        """Get a list of transcription jobs submitted within the last week in reverse
        chronological order up to the provided limit number of jobs per call.
        Pagination is supported via passing the last job id from previous call into starting_after.

        :param limit: optional, limits the number of jobs returned,
                      if none, a default of 100 jobs is returned, max limit if 1000
        :param starting_after: optional, returns jobs created after the job with this id,
                               exclusive (job with this id is not included)
        :returns: list of jobs response data
        :raises: HTTPError
        """
        params = []
        if limit is not None:
            params.append('limit={}'.format(limit))
        if starting_after is not None:
            params.append('starting_after={}'.format(starting_after))

        query = '?{}'.format('&'.join(params))
        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs{}'.format(query))
        )

        return [Job.from_json(job) for job in response.json()]

    def get_transcript_text(self, id_):
        """Get the transcript of a specific job as plain text.

        :param id_: id of job to be requested
        :returns: transcript data as text
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}/transcript'.format(id_)),
            headers={'Accept': 'text/plain'}
        )

        return response.text

    def get_transcript_text_as_stream(self, id_):
        """Get the transcript of a specific job as a plain text stream.

        :param id_: id of job to be requested
        :returns: requests.models.Response HTTP response which can be used to stream
            the payload of the response
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}/transcript'.format(id_)),
            headers={'Accept': 'text/plain'},
            stream=True
        )

        return response

    def get_transcript_json(self, id_):
        """Get the transcript of a specific job as json.

        :param id_: id of job to be requested
        :returns: transcript data as json
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}/transcript'.format(id_)),
            headers={'Accept': self.rev_json_content_type}
        )

        return response.json()

    def get_transcript_json_as_stream(self, id_):
        """Get the transcript of a specific job as streamed json.

        :param id_: id of job to be requested
        :returns: requests.models.Response HTTP response which can be used to stream
            the payload of the response
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}/transcript'.format(id_)),
            headers={'Accept': self.rev_json_content_type},
            stream=True
        )

        return response

    def get_transcript_object(self, id_):
        """Get the transcript of a specific job as a python object`.

        :param id_: id of job to be requested
        :returns: transcript data as a python object
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{}/transcript'.format(id_)),
            headers={'Accept': self.rev_json_content_type}
        )

        return Transcript.from_json(response.json())

    def get_captions(self, id_, content_type=CaptionType.SRT, channel_id=None):
        """Get the captions output of a specific job and return it as plain text

        :param id_: id of job to be requested
        :param content_type: caption type which should be returned. Defaults to SRT
        :param channel_id: id of speaker channel to be captioned, only matters for multichannel jobs
        :returns: caption data as text
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')
        query = self._create_captions_query(channel_id)

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{0}/captions{1}'.format(id_, query)),
            headers={'Accept': content_type.value}
        )

        return response.text

    def get_captions_as_stream(self, id_, content_type=CaptionType.SRT, channel_id=None):
        """Get the captions output of a specific job and return it as a plain text stream

        :param id_: id of job to be requested
        :param content_type: caption type which should be returned. Defaults to SRT
        :param channel_id: id of speaker channel to be captioned, only matters for multichannel jobs
        :returns: requests.models.Response HTTP response which can be used to stream
            the payload of the response
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')
        query = self._create_captions_query(channel_id)

        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'jobs/{0}/captions{1}'.format(id_, query)),
            headers={'Accept': content_type.value},
            stream=True
        )

        return response

    def delete_job(self, id_):
        """Delete a specific transcription job
        All data related to the job, such as input media and transcript, will be permanently
        deleted. A job can only by deleted once it's completed.

        :param id_: id of job to be deleted
        :returns: None if job was successfully deleted
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        self._make_http_request(
            "DELETE",
            urljoin(self.base_url, 'jobs/{}'.format(id_)),
        )

        return

    def get_account(self):
        """Get account information, such as remaining credits.

        :raises: HTTPError
        """
        response = self._make_http_request(
            "GET",
            urljoin(self.base_url, 'account')
        )

        return Account.from_json(response.json())

    def _create_job_options_payload(
            self,
            media_url=None,
            metadata=None,
            callback_url=None,
            skip_diarization=None,
            skip_punctuation=None,
            speaker_channels_count=None,
            custom_vocabularies=None,
            filter_profanity=None,
            remove_disfluencies=None,
            delete_after_seconds=None,
            language=None,
            custom_vocabulary_id=None,
            transcriber=None,
            verbatim=None,
            rush=None,
            test_mode=None,
            segments_to_transcribe=None,
            speaker_names=None,
            source_config=None,
            notification_config=None):
        payload = {}
        if media_url:
            payload['media_url'] = media_url
        if skip_diarization:
            payload['skip_diarization'] = skip_diarization
        if skip_punctuation:
            payload['skip_punctuation'] = skip_punctuation
        if metadata:
            payload['metadata'] = metadata
        if callback_url:
            payload['callback_url'] = callback_url
        if custom_vocabularies:
            payload['custom_vocabularies'] = utils._process_vocabularies(custom_vocabularies)
        if speaker_channels_count:
            payload['speaker_channels_count'] = speaker_channels_count
        if filter_profanity:
            payload['filter_profanity'] = filter_profanity
        if remove_disfluencies:
            payload['remove_disfluencies'] = remove_disfluencies
        if delete_after_seconds is not None:
            payload['delete_after_seconds'] = delete_after_seconds
        if language:
            payload['language'] = language
        if custom_vocabulary_id:
            payload['custom_vocabulary_id'] = custom_vocabulary_id
        if transcriber:
            payload['transcriber'] = transcriber
        if verbatim:
            payload['verbatim'] = verbatim
        if rush:
            payload['rush'] = rush
        if test_mode:
            payload['test_mode'] = test_mode
        if segments_to_transcribe:
            payload['segments_to_transcribe'] = segments_to_transcribe
        if speaker_names:
            payload['speaker_names'] =\
                utils._process_speaker_names(speaker_names)
        if source_config:
            payload['source_config'] = source_config.to_dict()
        if notification_config:
            payload['notification_config'] = notification_config.to_dict()
        return payload

    def _create_captions_query(self, speaker_channel):
        return '' if speaker_channel is None else '?speaker_channel={}'.format(speaker_channel)



# -*- coding: utf-8 -*-
"""Enum for Job statuses"""

from enum import Enum


class JobStatus(Enum):
    IN_PROGRESS = 1
    TRANSCRIBED = 2
    FAILED = 3
    COMPLETED = 4

    @classmethod
    def from_string(cls, status):
        return cls[status.upper()]



# -*- coding: utf-8 -*-
"""Sentiment Analysis Models"""

from .sentiment_analysis_job import SentimentAnalysisJob
from .sentiment_analysis_result import SentimentAnalysisResult, SentimentMessage
from .sentiment_value import SentimentValue



# -*- coding: utf-8 -*-
"""Account model"""


class Account:
    def __init__(self, email, balance_seconds):
        """
        :param email: email associated with the account
        :param balance_seconds: seconds of audio this account has the credits to transcribe
        """
        self.email = email
        self.balance_seconds = balance_seconds

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(json['email'], json['balance_seconds'])



# -*- coding: utf-8 -*-
"""Unit tests for job endpoints"""

import json
import pytest

from src.rev_ai.models.customer_url_data import CustomerUrlData
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models.asynchronous import Job, JobStatus, SpeakerName

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'
TOKEN = "token"
METADATA = 'test'
NOTIFICATION_URL = 'https://example.com/'
NOTIFICATION_AUTH = 'notification auth headers'
CREATED_ON = '2018-05-05T23:23:22.29Z'
SOURCE_URL = 'https://example.com/test.mp3'
SOURCE_AUTH = 'source auth headers'
FILENAME = 'test.mp3'
JOB_ID_URL = urljoin(RevAiAPIClient.base_url, 'jobs/{}'.format(JOB_ID))
JOBS_URL = urljoin(RevAiAPIClient.base_url, 'jobs')
CUSTOM_VOCAB = [{"phrases": ["word one", "word two"]}]
CUSTOM_VOCAB_ID = "vid"
LANGUAGE = 'en'
TRANSCRIBER = 'machine_v2'

SOURCE_CONFIG = CustomerUrlData(SOURCE_URL, SOURCE_AUTH)
NOTIFICATION_CONFIG = CustomerUrlData(NOTIFICATION_URL, NOTIFICATION_AUTH)


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestJobEndpoints():
    def test_get_job_details_with_success(self, mock_session, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            'id': JOB_ID,
            'status': status,
            'created_on': created_on
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.get_job_details(JOB_ID)

        assert res == Job(JOB_ID, created_on, JobStatus.TRANSCRIBED)
        mock_session.request.assert_called_once_with("GET",
                                                     JOB_ID_URL,
                                                     headers=client.default_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_job_details_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_job_details(id)

    def test_get_list_of_jobs_limit_with_success(self, mock_session, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = [
            {
                'id': JOB_ID,
                'status': status,
                'created_on': created_on
            },
            {
                'id': '2',
                'status': 'in_progress',
                'created_on': created_on
            }
        ]
        url = JOBS_URL + "?limit=2"
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.get_list_of_jobs(limit=2)

        assert isinstance(res, list)
        assert len(res) == 2
        mock_session.request.assert_called_once_with("GET", url, headers=client.default_headers)

    def test_get_list_of_jobs_starting_after_with_success(self, mock_session, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = [
            {
                'id': JOB_ID,
                'status': status,
                'created_on': created_on
            }
        ]
        url = JOBS_URL + "?starting_after=4"
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.get_list_of_jobs(starting_after="4")

        assert isinstance(res, list)
        assert len(res) == 1
        mock_session.request.assert_called_once_with("GET", url, headers=client.default_headers)

    def test_submit_job_url_with_success(self, mock_session, make_mock_response):
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'skip_diarization': True,
            'skip_punctuation': True,
            'speaker_channels_count': 1,
            'filter_profanity': True,
            'remove_disfluencies': True,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
            'transcriber': TRANSCRIBER
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.submit_job_url(SOURCE_URL, METADATA,
                                    NOTIFICATION_URL, True,
                                    True, 1, CUSTOM_VOCAB, True,
                                    True, 0, LANGUAGE, CUSTOM_VOCAB_ID,
                                    TRANSCRIBER)

        assert res == Job(JOB_ID,
                          CREATED_ON,
                          JobStatus.IN_PROGRESS,
                          metadata=METADATA,
                          skip_punctuation=True,
                          skip_diarization=True,
                          speaker_channels_count=1,
                          filter_profanity=True,
                          remove_disfluencies=True,
                          delete_after_seconds=0,
                          language=LANGUAGE,
                          transcriber=TRANSCRIBER)
        mock_session.request.assert_called_once_with(
            "POST",
            JOBS_URL,
            json={
                'media_url': SOURCE_URL,
                'callback_url': NOTIFICATION_URL,
                'metadata': METADATA,
                'skip_diarization': True,
                'skip_punctuation': True,
                'speaker_channels_count': 1,
                'custom_vocabularies': CUSTOM_VOCAB,
                'filter_profanity': True,
                'remove_disfluencies': True,
                'delete_after_seconds': 0,
                'language': LANGUAGE,
                'custom_vocabulary_id': CUSTOM_VOCAB_ID,
                'transcriber': TRANSCRIBER
            },
            headers=client.default_headers)

    def test_submit_job_url_with_auth_options(self, mock_session, make_mock_response):
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'skip_diarization': True,
            'skip_punctuation': True,
            'speaker_channels_count': 1,
            'filter_profanity': True,
            'remove_disfluencies': True,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
            'transcriber': TRANSCRIBER
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.submit_job_url(metadata=METADATA, skip_diarization=True, skip_punctuation=True,
                                    speaker_channels_count=1, custom_vocabularies=CUSTOM_VOCAB,
                                    filter_profanity=True, remove_disfluencies=True,
                                    delete_after_seconds=0, language=LANGUAGE,
                                    custom_vocabulary_id=CUSTOM_VOCAB_ID, transcriber=TRANSCRIBER,
                                    source_config=SOURCE_CONFIG,
                                    notification_config=NOTIFICATION_CONFIG)

        assert res == Job(JOB_ID,
                          CREATED_ON,
                          JobStatus.IN_PROGRESS,
                          metadata=METADATA,
                          skip_punctuation=True,
                          skip_diarization=True,
                          speaker_channels_count=1,
                          filter_profanity=True,
                          remove_disfluencies=True,
                          delete_after_seconds=0,
                          language=LANGUAGE,
                          transcriber=TRANSCRIBER)
        mock_session.request.assert_called_once_with(
            "POST",
            JOBS_URL,
            json={
                'source_config': {'url': SOURCE_URL, 'auth_headers': SOURCE_AUTH},
                'notification_config': {'url': NOTIFICATION_URL, 'auth_headers': NOTIFICATION_AUTH},
                'metadata': METADATA,
                'skip_diarization': True,
                'skip_punctuation': True,
                'speaker_channels_count': 1,
                'custom_vocabularies': CUSTOM_VOCAB,
                'filter_profanity': True,
                'remove_disfluencies': True,
                'delete_after_seconds': 0,
                'language': LANGUAGE,
                'custom_vocabulary_id': CUSTOM_VOCAB_ID,
                'transcriber': TRANSCRIBER
            },
            headers=client.default_headers)

    def test_submit_job_url_with_human_transcription_and_success(self, mock_session,
                                                                 make_mock_response):
        segments = [{
            'start': 1.0,
            'end': 2.0
        }]
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'transcriber': 'human',
            'verbatim': True,
            'segments_to_transcribe': segments
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.submit_job_url(SOURCE_URL, transcriber='human', verbatim=True, rush=False,
                                    segments_to_transcribe=segments,
                                    speaker_names=[SpeakerName('Kyle Bridburg')])

        assert res == Job(JOB_ID,
                          CREATED_ON,
                          JobStatus.IN_PROGRESS,
                          transcriber='human',
                          verbatim=True,
                          segments_to_transcribe=segments)
        mock_session.request.assert_called_once_with(
            'POST',
            JOBS_URL,
            json={
                'media_url': SOURCE_URL,
                'transcriber': 'human',
                'verbatim': True,
                'segments_to_transcribe': segments,
                'speaker_names': [{'display_name': 'Kyle Bridburg'}]
            },
            headers=client.default_headers)

    def test_submit_job_local_file_with_success(self, mocker, mock_session, make_mock_response):
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': created_on,
            'metadata': METADATA,
            'callback_url': NOTIFICATION_URL,
            'skip_punctuation': True,
            'skip_diarization': True,
            'speaker_channels_count': 1,
            'filter_profanity': True,
            'remove_disfluencies': True,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
            'transcriber': TRANSCRIBER
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as file:
            res = client.submit_job_local_file(FILENAME, METADATA,
                                               NOTIFICATION_URL, True,
                                               True, 1, CUSTOM_VOCAB, True,
                                               True, 0, LANGUAGE, CUSTOM_VOCAB_ID,
                                               TRANSCRIBER)

            assert res == Job(JOB_ID,
                              CREATED_ON,
                              JobStatus.IN_PROGRESS,
                              metadata=METADATA,
                              callback_url=NOTIFICATION_URL,
                              skip_punctuation=True,
                              skip_diarization=True,
                              speaker_channels_count=1,
                              filter_profanity=True,
                              remove_disfluencies=True,
                              delete_after_seconds=0,
                              language=LANGUAGE,
                              transcriber=TRANSCRIBER)
            mock_session.request.assert_called_once_with(
                "POST",
                JOBS_URL,
                files={
                    'media': (FILENAME, file),
                    'options': (
                        None,
                        json.dumps({
                            'metadata': METADATA,
                            'callback_url': NOTIFICATION_URL,
                            'skip_punctuation': True,
                            'skip_diarization': True,
                            'speaker_channels_count': 1,
                            'custom_vocabularies': CUSTOM_VOCAB,
                            'filter_profanity': True,
                            'remove_disfluencies': True,
                            'delete_after_seconds': 0,
                            'language': LANGUAGE,
                            'custom_vocabulary_id': CUSTOM_VOCAB_ID,
                            'transcriber': TRANSCRIBER
                        }, sort_keys=True)
                    )
                },
                headers=client.default_headers)

    def test_submit_job_local_file_auth_options_with_success(self, mocker, mock_session,
                                                             make_mock_response):
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': created_on,
            'metadata': METADATA,
            'skip_punctuation': True,
            'skip_diarization': True,
            'speaker_channels_count': 1,
            'filter_profanity': True,
            'remove_disfluencies': True,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
            'transcriber': TRANSCRIBER
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as file:
            res = client.submit_job_local_file(filename=FILENAME, metadata=METADATA,
                                               callback_url=None, skip_diarization=True,
                                               skip_punctuation=True, speaker_channels_count=1,
                                               custom_vocabularies=CUSTOM_VOCAB,
                                               filter_profanity=True, remove_disfluencies=True,
                                               delete_after_seconds=0, language=LANGUAGE,
                                               custom_vocabulary_id=CUSTOM_VOCAB_ID,
                                               transcriber=TRANSCRIBER,
                                               notification_config=NOTIFICATION_CONFIG)

            assert res == Job(JOB_ID,
                              CREATED_ON,
                              JobStatus.IN_PROGRESS,
                              metadata=METADATA,
                              skip_punctuation=True,
                              skip_diarization=True,
                              speaker_channels_count=1,
                              filter_profanity=True,
                              remove_disfluencies=True,
                              delete_after_seconds=0,
                              language=LANGUAGE,
                              transcriber=TRANSCRIBER)
            mock_session.request.assert_called_once_with(
                "POST",
                JOBS_URL,
                files={
                    'media': (FILENAME, file),
                    'options': (
                        None,
                        json.dumps({
                            'metadata': METADATA,
                            'notification_config': {'url': NOTIFICATION_URL,
                                                    'auth_headers': NOTIFICATION_AUTH},
                            'skip_punctuation': True,
                            'skip_diarization': True,
                            'speaker_channels_count': 1,
                            'custom_vocabularies': CUSTOM_VOCAB,
                            'filter_profanity': True,
                            'remove_disfluencies': True,
                            'delete_after_seconds': 0,
                            'language': LANGUAGE,
                            'custom_vocabulary_id': CUSTOM_VOCAB_ID,
                            'transcriber': TRANSCRIBER
                        }, sort_keys=True)
                    )
                },
                headers=client.default_headers)

    @pytest.mark.parametrize('filename', [None, ''])
    def test_submit_job_url_with_no_filename(self, filename, mock_session):
        with pytest.raises(ValueError, match='filename must be provided'):
            RevAiAPIClient(TOKEN).submit_job_local_file(filename, None)

    def test_delete_job_success(self, mock_session, make_mock_response):
        response = make_mock_response(url=JOB_ID_URL, status=204)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        res = client.delete_job(JOB_ID)

        assert res is None
        mock_session.request.assert_called_once_with("DELETE",
                                                     JOB_ID_URL,
                                                     headers=client.default_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_delete_job_with_no_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).delete_job(id)



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev AI"""

from . import CustomVocabulary
from . import SpeakerName


def _process_vocabularies(unprocessed_vocabularies):
    """
    This method takes in a list that contains CustomVocabulary objects
    and returns a list in which any such objects are converted properly
    to custom vocabulary dictionaries. Any items of other types in the list
    are not changed.
    """
    return list(map(lambda custom_vocabulary: custom_vocabulary.to_dict()
                    if isinstance(custom_vocabulary, CustomVocabulary)
                    else custom_vocabulary, unprocessed_vocabularies))


def _process_speaker_names(unprocessed_speaker_names):
    """
    This method takes in a list that contains SpeakerName objects
    and returns a list in which any such objects are converted properly
    to speaker name dictionaries. Any items of other types in the list
    are not changed.
    """
    return list(map(lambda speaker_name: speaker_name.to_dict()
                    if isinstance(speaker_name, SpeakerName)
                    else speaker_name, unprocessed_speaker_names))



# -*- coding: utf-8 -*-
"""Unit tests for the streaming client"""

import pytest
import six
from src.rev_ai import __version__
from src.rev_ai.models.streaming import MediaConfig
from src.rev_ai.streamingclient import RevAiStreamingClient

try:
    from urllib.parse import parse_qs, urlparse
except ImportError:
    from urlparse import parse_qs, urlparse


@pytest.mark.usefixtures('mock_streaming_client', 'mock_generator')
class TestStreamingClient():
    def test_constructor(self):
        example_token = 'token'
        example_config = MediaConfig()
        example_version = 'example_version'
        example_error_func = lambda example_error: example_error
        example_close_func = lambda code, reason: '{}:{}'.format(code, reason)
        example_connect_func = lambda id: id
        example_client = RevAiStreamingClient(
            example_token,
            example_config,
            example_version,
            example_error_func,
            example_close_func,
            example_connect_func
        )

        assert example_client.on_error("Example Error") == 'Example Error'
        assert example_client.on_connected("Example ID") == 'Example ID'
        assert example_client.on_close('1', 'Example Reason') == '1:Example Reason'
        assert example_client.base_url == 'wss://api.rev.ai/speechtotext/example_version/stream'

    def test_constructor_using_defaults(self):
        example_token = 'token'
        example_config = MediaConfig()
        example_client = RevAiStreamingClient(example_token, example_config)

        assert example_client.access_token == 'token'
        assert example_client.config == example_config

    def test_constructor_no_token_no_config(self):
        example_token = 'token'
        example_config = MediaConfig()

        with pytest.raises(ValueError):
            RevAiStreamingClient(example_token, None)
        with pytest.raises(ValueError):
            RevAiStreamingClient(None, example_config)

    def test_start_noparams_success(self, mock_streaming_client, mock_generator, capsys):
        expected_query_dict = build_expected_query_dict(mock_streaming_client, None, None, None, None, None, None, None,
            None)

        example_data = '{"type":"partial","transcript":"Test"}'
        example_connected = '{"type":"connected","id":"testid"}'
        if six.PY3:
            example_data = example_data.encode('utf-8')
            example_connected = example_connected.encode('utf-8')
        data = [[0x1, example_connected],
                [0x1, example_data],
                [0x8, b'\x03\xe8End of input. Closing']]
        exp_responses = ['Connected, Job ID : testid\n',
                         '{"type":"partial","transcript":"Test"}',
                         'Connection Closed. Code : 1000; Reason : End of input. Closing\n']
        mock_streaming_client.client.recv_data.side_effect = data

        response_gen = mock_streaming_client.start(mock_generator())

        called_url = mock_streaming_client.client.connect.call_args_list[0][0][0]
        validate_query_parameters(called_url, expected_query_dict)
        assert mock_streaming_client.client.connect.call_count == 1
        mock_streaming_client.client.send_binary.assert_any_call(0)
        mock_streaming_client.client.send_binary.assert_any_call(1)
        mock_streaming_client.client.send_binary.assert_any_call(2)
        assert hasattr(mock_streaming_client, 'request_thread')
        for ind, response in enumerate(response_gen):
            assert capsys.readouterr().out == exp_responses[ind]
            assert exp_responses[ind + 1] == response
        assert capsys.readouterr().out == exp_responses[2]

    @pytest.mark.parametrize("metadata", ["my metadata"])
    @pytest.mark.parametrize("custom_vocabulary_id", ["customvocabid"])
    @pytest.mark.parametrize("filter_profanity", [True])
    @pytest.mark.parametrize("remove_disfluencies", [True])
    @pytest.mark.parametrize("delete_after_seconds", [0])
    @pytest.mark.parametrize("detailed_partials", [True])
    @pytest.mark.parametrize("start_ts", [10])
    @pytest.mark.parametrize("transcriber", ["machine"])
    def test_start_allparams_success(self, mock_streaming_client, mock_generator, capsys,
        metadata, custom_vocabulary_id, filter_profanity, remove_disfluencies, delete_after_seconds, detailed_partials,
        start_ts, transcriber):

        expected_query_dict = build_expected_query_dict(
            mock_streaming_client,
            metadata,
            custom_vocabulary_id,
            filter_profanity,
            remove_disfluencies,
            delete_after_seconds,
            detailed_partials,
            start_ts,
            transcriber
        )
        example_data = '{"type":"partial","transcript":"Test"}'
        example_connected = '{"type":"connected","id":"testid"}'
        if six.PY3:
            example_data = example_data.encode('utf-8')
            example_connected = example_connected.encode('utf-8')
        data = [[0x1, example_connected],
                [0x1, example_data],
                [0x8, b'\x03\xe8End of input. Closing']]
        exp_responses = ['Connected, Job ID : testid\n',
                         '{"type":"partial","transcript":"Test"}',
                         'Connection Closed. Code : 1000; Reason : End of input. Closing\n']
        mock_streaming_client.client.recv_data.side_effect = data

        response_gen = mock_streaming_client.start(mock_generator(),
            metadata, custom_vocabulary_id, filter_profanity, remove_disfluencies, delete_after_seconds,
            detailed_partials, start_ts, transcriber)

        called_url = mock_streaming_client.client.connect.call_args_list[0][0][0]
        validate_query_parameters(called_url, expected_query_dict)
        assert mock_streaming_client.client.connect.call_count == 1
        mock_streaming_client.client.send_binary.assert_any_call(0)
        mock_streaming_client.client.send_binary.assert_any_call(1)
        mock_streaming_client.client.send_binary.assert_any_call(2)
        assert hasattr(mock_streaming_client, 'request_thread')
        for ind, response in enumerate(response_gen):
            assert capsys.readouterr().out == exp_responses[ind]
            assert exp_responses[ind + 1] == response
        assert capsys.readouterr().out == exp_responses[2]

    def test_start_failure_to_connect(self, mock_streaming_client, mock_generator):
        mock_streaming_client.client.connect = lambda x: 1 / 0

        with pytest.raises(ZeroDivisionError):
            mock_streaming_client.start(mock_generator())

    def test_end(self, mock_streaming_client):
        mock_streaming_client.end()

        mock_streaming_client.client.abort.assert_called_once_with()


def build_expected_query_dict(mock_streaming_client,
    metadata, custom_vocabulary_id, filter_profanity, remove_disfluencies, delete_after_seconds, detailed_partials,
    start_ts, transcriber):
    expected_query_dict = {
        'access_token': mock_streaming_client.access_token,
        'content_type': mock_streaming_client.config.get_content_type_string(),
        'user_agent': 'RevAi-PythonSDK/{}'.format(__version__),
    }

    if metadata:
        expected_query_dict["metadata"] = metadata
    if custom_vocabulary_id:
        expected_query_dict["custom_vocabulary_id"] = custom_vocabulary_id
    if filter_profanity:
        expected_query_dict["filter_profanity"] = "true"
    if remove_disfluencies:
        expected_query_dict["remove_disfluencies"] = "true"
    if delete_after_seconds:
        expected_query_dict["delete_after_seconds"] = str(delete_after_seconds)
    if detailed_partials:
        expected_query_dict["detailed_partials"] = "true"
    if start_ts:
        expected_query_dict["start_ts"] = str(start_ts)
    if transcriber:
        expected_query_dict["transcriber"] = transcriber

    return expected_query_dict


def validate_query_parameters(called_url, expected_query_dict):
    called_query_string = urlparse(called_url).query
    called_query_parameters = parse_qs(called_query_string)
    for key in expected_query_dict:
        assert called_query_parameters[key][0] == expected_query_dict[key]
